@@ -53,8 +53,9 @@ const (
 )
 
 const (
-	SystemSecretSystemAppSecretName             = "system-app"
-	SystemSecretSystemAppSecretKeyBaseFieldName = "SECRET_KEY_BASE"
+	SystemSecretSystemAppSecretName              = "system-app"
+	SystemSecretSystemAppSecretKeyBaseFieldName  = "SECRET_KEY_BASE"
+	SystemSecretSystemAppUserSessionTTLFieldName = "USER_SESSION_TTL"
 )
 
 const (
@@ -85,6 +86,7 @@ const (
 	SystemSecretSystemSMTPPortFieldName              = "port"
 	SystemSecretSystemSMTPAuthenticationFieldName    = "authentication"
 	SystemSecretSystemSMTPOpenSSLVerifyModeFieldName = "openssl.verify.mode"
+	SystemSecretSystemSMTPFromAddressFieldName       = "from_address"
 )
 
 const (
@@ -92,8 +94,9 @@ const (
 )
 
 const (
-	SystemSidekiqName       = "system-sidekiq"
-	SystemAppDeploymentName = "system-app"
+	SystemSidekiqName          = "system-sidekiq"
+	SystemAppDeploymentName    = "system-app"
+	SystemSphinxDeploymentName = "system-sphinx"
 
 	SystemAppMasterContainerName    = "system-master"
 	SystemAppProviderContainerName  = "system-provider"
@@ -140,13 +143,18 @@ func (system *System) getSystemBaseEnvsFromEnvConfigMap() []v1.EnvVar {
 
 func (system *System) getSystemSMTPEnvsFromSMTPSecret() []v1.EnvVar {
 	result := []v1.EnvVar{
-		helper.EnvVarFromSecret("SMTP_ADDRESS", SystemSecretSystemSMTPSecretName, "address"),
-		helper.EnvVarFromSecret("SMTP_USER_NAME", SystemSecretSystemSMTPSecretName, "username"),
-		helper.EnvVarFromSecret("SMTP_PASSWORD", SystemSecretSystemSMTPSecretName, "password"),
-		helper.EnvVarFromSecret("SMTP_DOMAIN", SystemSecretSystemSMTPSecretName, "domain"),
-		helper.EnvVarFromSecret("SMTP_PORT", SystemSecretSystemSMTPSecretName, "port"),
-		helper.EnvVarFromSecret("SMTP_AUTHENTICATION", SystemSecretSystemSMTPSecretName, "authentication"),
-		helper.EnvVarFromSecret("SMTP_OPENSSL_VERIFY_MODE", SystemSecretSystemSMTPSecretName, "openssl.verify.mode"),
+		helper.EnvVarFromSecret("SMTP_ADDRESS", SystemSecretSystemSMTPSecretName, SystemSecretSystemSMTPAddressFieldName),
+		helper.EnvVarFromSecret("SMTP_USER_NAME", SystemSecretSystemSMTPSecretName, SystemSecretSystemSMTPUserNameFieldName),
+		helper.EnvVarFromSecret("SMTP_PASSWORD", SystemSecretSystemSMTPSecretName, SystemSecretSystemSMTPPasswordFieldName),
+		helper.EnvVarFromSecret("SMTP_DOMAIN", SystemSecretSystemSMTPSecretName, SystemSecretSystemSMTPDomainFieldName),
+		helper.EnvVarFromSecret("SMTP_PORT", SystemSecretSystemSMTPSecretName, SystemSecretSystemSMTPPortFieldName),
+		helper.EnvVarFromSecret("SMTP_AUTHENTICATION", SystemSecretSystemSMTPSecretName, SystemSecretSystemSMTPAuthenticationFieldName),
+		helper.EnvVarFromSecret("SMTP_OPENSSL_VERIFY_MODE", SystemSecretSystemSMTPSecretName, SystemSecretSystemSMTPOpenSSLVerifyModeFieldName),
+	}
+
+	if system.Options.SmtpSecretOptions.FromAddress != nil &&
+		*system.Options.SmtpSecretOptions.FromAddress != "" {
+		result = append(result, helper.EnvVarFromSecret("NOREPLY_EMAIL", SystemSecretSystemSMTPSecretName, SystemSecretSystemSMTPFromAddressFieldName))
 	}
 
 	return result
@@ -219,8 +227,8 @@ func (system *System) buildSystemBaseEnv() []v1.EnvVar {
 
 	result = append(result, system.SystemRedisEnvVars()...)
 	result = append(result, system.BackendRedisEnvVars()...)
-	bckListenerApicastRouteEnv := helper.EnvVarFromSecret("APICAST_BACKEND_ROOT_ENDPOINT", "backend-listener", "route_endpoint")
-	bckListenerRouteEnv := helper.EnvVarFromSecret("BACKEND_ROUTE", "backend-listener", "route_endpoint")
+	bckListenerApicastRouteEnv := helper.EnvVarFromSecret("APICAST_BACKEND_ROOT_ENDPOINT", BackendSecretBackendListenerSecretName, BackendSecretBackendListenerRouteEndpointFieldName)
+	bckListenerRouteEnv := helper.EnvVarFromValue("BACKEND_ROUTE", system.Options.BackendServiceEndpoint)
 	result = append(result, bckListenerApicastRouteEnv, bckListenerRouteEnv)
 
 	smtpEnvSecretEnvs := system.getSystemSMTPEnvsFromSMTPSecret()
@@ -254,11 +262,18 @@ func (system *System) buildSystemBaseEnv() []v1.EnvVar {
 	return result
 }
 
+func (system *System) buildAppEnv() []v1.EnvVar {
+	result := []v1.EnvVar{}
+	result = append(result, helper.EnvVarFromSecret(SystemSecretSystemAppUserSessionTTLFieldName, SystemSecretSystemAppSecretName, SystemSecretSystemAppUserSessionTTLFieldName))
+	return result
+}
+
 func (system *System) buildAppMasterContainerEnv() []v1.EnvVar {
 	result := system.buildSystemBaseEnv()
 	if system.Options.AppMetrics {
 		result = append(result, helper.EnvVarFromValue(SystemAppPrometheusExporterPortEnvVarName, strconv.Itoa(SystemAppMasterContainerPrometheusPort)))
 	}
+	result = append(result, system.buildAppEnv()...)
 
 	return result
 }
@@ -268,6 +283,8 @@ func (system *System) buildAppProviderContainerEnv() []v1.EnvVar {
 	if system.Options.AppMetrics {
 		result = append(result, helper.EnvVarFromValue(SystemAppPrometheusExporterPortEnvVarName, strconv.Itoa(SystemAppProviderContainerPrometheusPort)))
 	}
+	result = append(result, system.buildAppEnv()...)
+
 	return result
 }
 
@@ -276,6 +293,7 @@ func (system *System) buildAppDeveloperContainerEnv() []v1.EnvVar {
 	if system.Options.AppMetrics {
 		result = append(result, helper.EnvVarFromValue(SystemAppPrometheusExporterPortEnvVarName, strconv.Itoa(SystemAppDeveloperContainerPrometheusPort)))
 	}
+	result = append(result, system.buildAppEnv()...)
 
 	return result
 }
@@ -404,7 +422,7 @@ func (system *System) EventsHookSecret() *v1.Secret {
 }
 
 func (system *System) AppSecret() *v1.Secret {
-	return &v1.Secret{
+	result := &v1.Secret{
 		TypeMeta: metav1.TypeMeta{
 			APIVersion: "v1",
 			Kind:       "Secret",
@@ -414,10 +432,13 @@ func (system *System) AppSecret() *v1.Secret {
 			Labels: system.Options.CommonLabels,
 		},
 		StringData: map[string]string{
-			SystemSecretSystemAppSecretKeyBaseFieldName: system.Options.AppSecretKeyBase,
+			SystemSecretSystemAppSecretKeyBaseFieldName:  system.Options.AppSecretKeyBase,
+			SystemSecretSystemAppUserSessionTTLFieldName: *system.Options.UserSessionTTL,
 		},
 		Type: v1.SecretTypeOpaque,
 	}
+
+	return result
 }
 
 func (system *System) SeedSecret() *v1.Secret {
@@ -1042,7 +1063,7 @@ func (system *System) MemcachedService() *v1.Service {
 }
 
 func (system *System) SMTPSecret() *v1.Secret {
-	return &v1.Secret{
+	res := &v1.Secret{
 		TypeMeta: metav1.TypeMeta{
 			Kind:       "Secret",
 			APIVersion: "v1",
@@ -1061,6 +1082,12 @@ func (system *System) SMTPSecret() *v1.Secret {
 			SystemSecretSystemSMTPUserNameFieldName:          *system.Options.SmtpSecretOptions.Username,
 		},
 	}
+
+	if system.Options.SmtpSecretOptions.FromAddress != nil {
+		res.StringData[SystemSecretSystemSMTPFromAddressFieldName] = *system.Options.SmtpSecretOptions.FromAddress
+	}
+
+	return res
 }
 
 func (system *System) SystemConfigMap() *v1.ConfigMap {
@@ -1123,7 +1150,7 @@ func (system *System) SphinxDeploymentConfig() *appsv1.DeploymentConfig {
 			APIVersion: "apps.openshift.io/v1",
 		},
 		ObjectMeta: metav1.ObjectMeta{
-			Name:   "system-sphinx",
+			Name:   SystemSphinxDeploymentName,
 			Labels: system.Options.SphinxLabels,
 		},
 		Spec: appsv1.DeploymentConfigSpec{
@@ -1147,7 +1174,7 @@ func (system *System) SphinxDeploymentConfig() *appsv1.DeploymentConfig {
 				},
 			},
 			Replicas: 1,
-			Selector: map[string]string{"deploymentConfig": "system-sphinx"},
+			Selector: map[string]string{"deploymentConfig": SystemSphinxDeploymentName},
 			Strategy: appsv1.DeploymentStrategy{
 				RollingParams: &appsv1.RollingDeploymentStrategyParams{
 					IntervalSeconds: &[]int64{1}[0],
