@@ -5,13 +5,17 @@ import (
 	"strconv"
 	"testing"
 
-	"github.com/3scale/3scale-operator/pkg/3scale/amp/component"
-	"github.com/3scale/3scale-operator/pkg/3scale/amp/product"
-	appsv1alpha1 "github.com/3scale/3scale-operator/pkg/apis/apps/v1alpha1"
 	"github.com/google/go-cmp/cmp"
 	"github.com/google/go-cmp/cmp/cmpopts"
 	v1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/resource"
+	"k8s.io/apimachinery/pkg/runtime"
+	"sigs.k8s.io/controller-runtime/pkg/client/fake"
+
+	appsv1alpha1 "github.com/3scale/3scale-operator/apis/apps/v1alpha1"
+	"github.com/3scale/3scale-operator/pkg/3scale/amp/component"
+	"github.com/3scale/3scale-operator/pkg/3scale/amp/product"
+	"github.com/3scale/3scale-operator/pkg/helper"
 )
 
 const (
@@ -52,9 +56,9 @@ func testApicastStagingPodLabels() map[string]string {
 		"threescale_component_element": "staging",
 		"com.redhat.component-name":    "apicast-staging",
 		"com.redhat.component-type":    "application",
-		"com.redhat.component-version": "2.10",
+		"com.redhat.component-version": helper.ParseVersion(ApicastImageURL()),
 		"com.redhat.product-name":      "3scale",
-		"com.redhat.product-version":   "2.10",
+		"com.redhat.product-version":   product.ThreescaleRelease,
 		"deploymentConfig":             "apicast-staging",
 	}
 }
@@ -66,9 +70,9 @@ func testApicastProductionPodLabels() map[string]string {
 		"threescale_component_element": "production",
 		"com.redhat.component-name":    "apicast-production",
 		"com.redhat.component-type":    "application",
-		"com.redhat.component-version": "2.10",
+		"com.redhat.component-version": helper.ParseVersion(ApicastImageURL()),
 		"com.redhat.product-name":      "3scale",
-		"com.redhat.product-version":   "2.10",
+		"com.redhat.product-version":   product.ThreescaleRelease,
 		"deploymentConfig":             "apicast-production",
 	}
 }
@@ -154,6 +158,8 @@ func defaultApicastOptions() *component.ApicastOptions {
 		StagingPodTemplateLabels:       testApicastStagingPodLabels(),
 		ProductionPodTemplateLabels:    testApicastProductionPodLabels(),
 		Namespace:                      namespace,
+		ProductionTracingConfig:        &component.APIcastTracingConfig{TracingLibrary: component.APIcastDefaultTracingLibrary},
+		StagingTracingConfig:           &component.APIcastTracingConfig{TracingLibrary: component.APIcastDefaultTracingLibrary},
 	}
 }
 
@@ -240,7 +246,9 @@ func TestGetApicastOptionsProvider(t *testing.T) {
 
 	for _, tc := range cases {
 		t.Run(tc.name, func(subT *testing.T) {
-			optsProvider := NewApicastOptionsProvider(tc.apimanagerFactory())
+			objs := []runtime.Object{}
+			cl := fake.NewFakeClient(objs...)
+			optsProvider := NewApicastOptionsProvider(tc.apimanagerFactory(), cl)
 			opts, err := optsProvider.GetApicastOptions()
 			if err != nil {
 				subT.Error(err)
