@@ -4,9 +4,10 @@ import (
 	"reflect"
 	"testing"
 
+	appsv1alpha1 "github.com/3scale/3scale-operator/apis/apps/v1alpha1"
 	"github.com/3scale/3scale-operator/pkg/3scale/amp/component"
 	"github.com/3scale/3scale-operator/pkg/3scale/amp/product"
-	appsv1alpha1 "github.com/3scale/3scale-operator/pkg/apis/apps/v1alpha1"
+	"github.com/3scale/3scale-operator/pkg/helper"
 
 	"github.com/google/go-cmp/cmp"
 	"github.com/google/go-cmp/cmp/cmpopts"
@@ -38,9 +39,9 @@ func testRedisSystemRedisPodTemplateLabels() map[string]string {
 		"threescale_component_element": "redis",
 		"com.redhat.component-name":    "system-redis",
 		"com.redhat.component-type":    "application",
-		"com.redhat.component-version": "3.2",
+		"com.redhat.component-version": helper.ParseVersion(SystemRedisImageURL()),
 		"com.redhat.product-name":      "3scale",
-		"com.redhat.product-version":   "2.10",
+		"com.redhat.product-version":   product.ThreescaleRelease,
 		"deploymentConfig":             "system-redis",
 	}
 }
@@ -67,9 +68,9 @@ func testRedisBackendRedisPodTemplateLabels() map[string]string {
 		"threescale_component_element": "redis",
 		"com.redhat.component-name":    "backend-redis",
 		"com.redhat.component-type":    "application",
-		"com.redhat.component-version": "3.2",
+		"com.redhat.component-version": helper.ParseVersion(BackendRedisImageURL()),
 		"com.redhat.product-name":      "3scale",
-		"com.redhat.product-version":   "2.10",
+		"com.redhat.product-version":   product.ThreescaleRelease,
 		"deploymentConfig":             "backend-redis",
 	}
 }
