@@ -6,9 +6,10 @@ import (
 	"strings"
 	"testing"
 
+	appsv1alpha1 "github.com/3scale/3scale-operator/apis/apps/v1alpha1"
 	"github.com/3scale/3scale-operator/pkg/3scale/amp/component"
 	"github.com/3scale/3scale-operator/pkg/3scale/amp/product"
-	appsv1alpha1 "github.com/3scale/3scale-operator/pkg/apis/apps/v1alpha1"
+	"github.com/3scale/3scale-operator/pkg/helper"
 
 	"github.com/google/go-cmp/cmp"
 	"github.com/google/go-cmp/cmp/cmpopts"
@@ -45,9 +46,9 @@ func testSystemMysqlPodTemplateLabels() map[string]string {
 		"threescale_component_element": "mysql",
 		"com.redhat.component-name":    "system-mysql",
 		"com.redhat.component-type":    "application",
-		"com.redhat.component-version": "5.7",
+		"com.redhat.component-version": helper.ParseVersion(SystemMySQLImageURL()),
 		"com.redhat.product-name":      "3scale",
-		"com.redhat.product-version":   "2.10",
+		"com.redhat.product-version":   product.ThreescaleRelease,
 		"deploymentConfig":             "system-mysql",
 	}
 }
