@@ -2,10 +2,11 @@ package operator
 
 import (
 	"fmt"
+	"net/url"
 
+	appsv1alpha1 "github.com/3scale/3scale-operator/apis/apps/v1alpha1"
 	"github.com/3scale/3scale-operator/pkg/3scale/amp/component"
 	"github.com/3scale/3scale-operator/pkg/3scale/amp/product"
-	appsv1alpha1 "github.com/3scale/3scale-operator/pkg/apis/apps/v1alpha1"
 	"github.com/3scale/3scale-operator/pkg/helper"
 
 	v1 "k8s.io/api/core/v1"
@@ -114,6 +115,11 @@ func (s *SystemOptionsProvider) setSecretBasedOptions() error {
 		return fmt.Errorf("unable to create System SMTP secret options - %s", err)
 	}
 
+	err = s.setBackendOptions()
+	if err != nil {
+		return err
+	}
+
 	return nil
 }
 
@@ -184,6 +190,15 @@ func (s *SystemOptionsProvider) setSystemAppOptions() error {
 	}
 	s.options.AppSecretKeyBase = val
 
+	val, err = s.secretSource.FieldValue(
+		component.SystemSecretSystemAppSecretName,
+		component.SystemSecretSystemAppUserSessionTTLFieldName,
+		component.DefaultUserSessionTTL())
+	if err != nil {
+		return err
+	}
+	s.options.UserSessionTTL = &val
+
 	return nil
 }
 
@@ -330,6 +345,12 @@ func (s *SystemOptionsProvider) setSystemSMTPOptions() error {
 			component.SystemSecretSystemSMTPUserNameFieldName,
 			component.DefaultSystemSMTPUsername(),
 		},
+		{
+			&smtpSecretOptions.FromAddress,
+			component.SystemSecretSystemSMTPSecretName,
+			component.SystemSecretSystemSMTPFromAddressFieldName,
+			component.DefaultSystemSMTPFromAddress(),
+		},
 	}
 
 	for _, option := range cases {
@@ -344,6 +365,29 @@ func (s *SystemOptionsProvider) setSystemSMTPOptions() error {
 	return nil
 }
 
+func (s *SystemOptionsProvider) setBackendOptions() error {
+	rawURL, err := s.secretSource.FieldValue(
+		component.BackendSecretBackendListenerSecretName,
+		component.BackendSecretBackendListenerServiceEndpointFieldName,
+		component.DefaultBackendServiceEndpoint())
+	if err != nil {
+		return err
+	}
+
+	urlObj, err := url.Parse(rawURL)
+	if err != nil {
+		return fmt.Errorf("'%s' field of '%s' secret must have 'scheme://user:password@host/path' format", component.BackendSecretBackendListenerServiceEndpointFieldName, component.BackendSecretBackendListenerSecretName)
+	}
+
+	if urlObj.Path == "" {
+		urlObj.Path = "/internal/"
+	}
+
+	s.options.BackendServiceEndpoint = urlObj.String()
+
+	return nil
+}
+
 func (s *SystemOptionsProvider) setResourceRequirementsOptions() {
 	if *s.apimanager.Spec.ResourceRequirementsEnabled {
 		s.options.AppMasterContainerResourceRequirements = component.DefaultAppMasterContainerResourceRequirements()
