@@ -1,12 +1,14 @@
 package operator
 
 import (
+	"fmt"
 	"reflect"
 	"testing"
 
+	appsv1alpha1 "github.com/3scale/3scale-operator/apis/apps/v1alpha1"
 	"github.com/3scale/3scale-operator/pkg/3scale/amp/component"
 	"github.com/3scale/3scale-operator/pkg/3scale/amp/product"
-	appsv1alpha1 "github.com/3scale/3scale-operator/pkg/apis/apps/v1alpha1"
+	"github.com/3scale/3scale-operator/pkg/helper"
 
 	"github.com/google/go-cmp/cmp"
 	"github.com/google/go-cmp/cmp/cmpopts"
@@ -44,9 +46,9 @@ func testSystemAppPodTemplateLabels() map[string]string {
 		"threescale_component_element": "app",
 		"com.redhat.component-name":    "system-app",
 		"com.redhat.component-type":    "application",
-		"com.redhat.component-version": "2.10",
+		"com.redhat.component-version": helper.ParseVersion(SystemImageURL()),
 		"com.redhat.product-name":      "3scale",
-		"com.redhat.product-version":   "2.10",
+		"com.redhat.product-version":   product.ThreescaleRelease,
 		"deploymentConfig":             "system-app",
 	}
 }
@@ -66,9 +68,9 @@ func testSystemSidekiqPodTemplateLabels() map[string]string {
 		"threescale_component_element": "sidekiq",
 		"com.redhat.component-name":    "system-sidekiq",
 		"com.redhat.component-type":    "application",
-		"com.redhat.component-version": "2.10",
+		"com.redhat.component-version": helper.ParseVersion(SystemImageURL()),
 		"com.redhat.product-name":      "3scale",
-		"com.redhat.product-version":   "2.10",
+		"com.redhat.product-version":   product.ThreescaleRelease,
 		"deploymentConfig":             "system-sidekiq",
 	}
 }
@@ -112,9 +114,9 @@ func testSystemSphinxPodTemplateLabels() map[string]string {
 		"threescale_component_element": "sphinx",
 		"com.redhat.component-name":    "system-sphinx",
 		"com.redhat.component-type":    "application",
-		"com.redhat.component-version": "2.10",
+		"com.redhat.component-version": helper.ParseVersion(SystemImageURL()),
 		"com.redhat.product-name":      "3scale",
-		"com.redhat.product-version":   "2.10",
+		"com.redhat.product-version":   product.ThreescaleRelease,
 		"deploymentConfig":             "system-sphinx",
 	}
 }
@@ -266,11 +268,19 @@ func getEventHookSecret() *v1.Secret {
 
 func getSystemAppSecret() *v1.Secret {
 	data := map[string]string{
-		component.SystemSecretSystemAppSecretKeyBaseFieldName: "somePassword234",
+		component.SystemSecretSystemAppSecretKeyBaseFieldName:  "somePassword234",
+		component.SystemSecretSystemAppUserSessionTTLFieldName: "3333",
 	}
 	return GetTestSecret(namespace, component.SystemSecretSystemAppSecretName, data)
 }
 
+func getSMTPSecretWithCustomSMTPAddress(address string) *v1.Secret {
+	data := map[string]string{
+		component.SystemSecretSystemSMTPFromAddressFieldName: address,
+	}
+	return GetTestSecret(namespace, component.SystemSecretSystemSMTPSecretName, data)
+}
+
 func getSystemSeedSecret() *v1.Secret {
 	data := map[string]string{
 		component.SystemSecretSystemSeedMasterDomainFieldName:      "masterDomainName",
@@ -298,6 +308,7 @@ func defaultSystemOptions(opts *component.SystemOptions) *component.SystemOption
 	tmpSystemAppReplicas := int32(systemAppReplicas)
 	tmpSystemSideKiqReplicas := int32(systemSidekiqReplicas)
 	tmpSystemAdminEmail := component.DefaultSystemAdminEmail()
+	tmpSystemUserSessionTTL := component.DefaultUserSessionTTL()
 	tmpSMTPAddress := component.DefaultSystemSMTPAddress()
 	tmpSMTPAuthentication := component.DefaultSystemSMTPAuthentication()
 	tmpSMTPDomain := component.DefaultSystemSMTPDomain()
@@ -305,6 +316,7 @@ func defaultSystemOptions(opts *component.SystemOptions) *component.SystemOption
 	tmpSMTPPassword := opts.SmtpSecretOptions.Password
 	tmpSMTPPort := component.DefaultSystemSMTPPort()
 	tmpSMTPUsername := component.DefaultSystemSMTPUsername()
+	tmpSMTPFromAddress := component.DefaultSystemSMTPFromAddress()
 	storageRequests := component.DefaultSharedStorageResources()
 
 	expectedOpts := &component.SystemOptions{
@@ -335,6 +347,7 @@ func defaultSystemOptions(opts *component.SystemOptions) *component.SystemOption
 		AppReplicas:                               &tmpSystemAppReplicas,
 		SidekiqReplicas:                           &tmpSystemSideKiqReplicas,
 		AdminEmail:                                &tmpSystemAdminEmail,
+		UserSessionTTL:                            &tmpSystemUserSessionTTL,
 		PvcFileStorageOptions: &component.PVCFileStorageOptions{
 			StorageRequests: storageRequests,
 		},
@@ -346,6 +359,7 @@ func defaultSystemOptions(opts *component.SystemOptions) *component.SystemOption
 			Password:          tmpSMTPPassword,
 			Port:              &tmpSMTPPort,
 			Username:          &tmpSMTPUsername,
+			FromAddress:       &tmpSMTPFromAddress,
 		},
 		CommonLabels:                  testSystemCommonLabels(),
 		CommonAppLabels:               testSystemCommonAppLabels(),
@@ -362,6 +376,7 @@ func defaultSystemOptions(opts *component.SystemOptions) *component.SystemOption
 		SideKiqMetrics:                true,
 		AppMetrics:                    true,
 		IncludeOracleOptionalSettings: true,
+		BackendServiceEndpoint:        fmt.Sprintf("%s%s", component.DefaultBackendServiceEndpoint(), "/internal/"),
 		Namespace:                     opts.Namespace,
 	}
 
@@ -381,10 +396,11 @@ func TestGetSystemOptionsProvider(t *testing.T) {
 		systemAppSecret           *v1.Secret
 		systemSeedSecret          *v1.Secret
 		systemMasterApicastSecret *v1.Secret
+		systemSMTPSecret          *v1.Secret
 		expectedOptionsFactory    func(*component.SystemOptions) *component.SystemOptions
 	}{
 		{"Default", basicApimanagerSpecTestSystemOptions,
-			nil, nil, nil, nil, nil, nil,
+			nil, nil, nil, nil, nil, nil, nil,
 			func(opts *component.SystemOptions) *component.SystemOptions {
 				return defaultSystemOptions(opts)
 			},
@@ -394,7 +410,7 @@ func TestGetSystemOptionsProvider(t *testing.T) {
 				apimanager := basicApimanagerSpecTestSystemOptions()
 				apimanager.Spec.ResourceRequirementsEnabled = &falseValue
 				return apimanager
-			}, nil, nil, nil, nil, nil, nil,
+			}, nil, nil, nil, nil, nil, nil, nil,
 			func(opts *component.SystemOptions) *component.SystemOptions {
 				expectedOpts := defaultSystemOptions(opts)
 				expectedOpts.AppMasterContainerResourceRequirements = &v1.ResourceRequirements{}
@@ -406,7 +422,7 @@ func TestGetSystemOptionsProvider(t *testing.T) {
 			},
 		},
 		{"WithMemcachedSecret", basicApimanagerSpecTestSystemOptions,
-			getMemcachedSecret(), nil, nil, nil, nil, nil,
+			getMemcachedSecret(), nil, nil, nil, nil, nil, nil,
 			func(opts *component.SystemOptions) *component.SystemOptions {
 				expectedOpts := defaultSystemOptions(opts)
 				expectedOpts.MemcachedServers = "mymemcache:11211"
@@ -414,7 +430,7 @@ func TestGetSystemOptionsProvider(t *testing.T) {
 			},
 		},
 		{"WithRecaptchaSecret", basicApimanagerSpecTestSystemOptions,
-			nil, getRecaptchaSecret(), nil, nil, nil, nil,
+			nil, getRecaptchaSecret(), nil, nil, nil, nil, nil,
 			func(opts *component.SystemOptions) *component.SystemOptions {
 				expectedOpts := defaultSystemOptions(opts)
 				recaptchaPublicKey := "someCaptchaPK"
@@ -425,7 +441,7 @@ func TestGetSystemOptionsProvider(t *testing.T) {
 			},
 		},
 		{"WithEventsHookSecret", basicApimanagerSpecTestSystemOptions,
-			nil, nil, getEventHookSecret(), nil, nil, nil,
+			nil, nil, getEventHookSecret(), nil, nil, nil, nil,
 			func(opts *component.SystemOptions) *component.SystemOptions {
 				expectedOpts := defaultSystemOptions(opts)
 				expectedOpts.BackendSharedSecret = "somePassword"
@@ -434,15 +450,17 @@ func TestGetSystemOptionsProvider(t *testing.T) {
 			},
 		},
 		{"WithAppSecret", basicApimanagerSpecTestSystemOptions,
-			nil, nil, nil, getSystemAppSecret(), nil, nil,
+			nil, nil, nil, getSystemAppSecret(), nil, nil, nil,
 			func(opts *component.SystemOptions) *component.SystemOptions {
 				expectedOpts := defaultSystemOptions(opts)
 				expectedOpts.AppSecretKeyBase = "somePassword234"
+				tmpUserSessionTTL := "3333"
+				expectedOpts.UserSessionTTL = &tmpUserSessionTTL
 				return expectedOpts
 			},
 		},
 		{"WithSeedSecret", basicApimanagerSpecTestSystemOptions,
-			nil, nil, nil, nil, getSystemSeedSecret(), nil,
+			nil, nil, nil, nil, getSystemSeedSecret(), nil, nil,
 			func(opts *component.SystemOptions) *component.SystemOptions {
 				expectedOpts := defaultSystemOptions(opts)
 				expectedOpts.MasterName = "masterDomainName"
@@ -458,7 +476,7 @@ func TestGetSystemOptionsProvider(t *testing.T) {
 			},
 		},
 		{"WithMasterApicastSecret", basicApimanagerSpecTestSystemOptions,
-			nil, nil, nil, nil, nil, getSystemMasterApicastSecret(),
+			nil, nil, nil, nil, nil, getSystemMasterApicastSecret(), nil,
 			func(opts *component.SystemOptions) *component.SystemOptions {
 				expectedOpts := defaultSystemOptions(opts)
 				expectedOpts.ApicastAccessToken = "apicastAccessToken"
@@ -475,7 +493,7 @@ func TestGetSystemOptionsProvider(t *testing.T) {
 				}
 				return apimanager
 			},
-			nil, nil, nil, nil, nil, nil,
+			nil, nil, nil, nil, nil, nil, nil,
 			func(opts *component.SystemOptions) *component.SystemOptions {
 				expectedOpts := defaultSystemOptions(opts)
 				expectedOpts.S3FileStorageOptions = &component.S3FileStorageOptions{
@@ -498,7 +516,7 @@ func TestGetSystemOptionsProvider(t *testing.T) {
 					VolumeName: &tmpVolumeName,
 				}
 				return apimanager
-			}, nil, nil, nil, nil, nil, nil,
+			}, nil, nil, nil, nil, nil, nil, nil,
 			func(opts *component.SystemOptions) *component.SystemOptions {
 				expectedOpts := defaultSystemOptions(opts)
 				tmp := "mystorageclassname"
@@ -517,7 +535,7 @@ func TestGetSystemOptionsProvider(t *testing.T) {
 				apimanager.Spec.System.SidekiqSpec.Affinity = testSystemSidekiqAffinity()
 				apimanager.Spec.System.SphinxSpec.Affinity = testSystemSphinxAffinity()
 				return apimanager
-			}, nil, nil, nil, nil, nil, nil,
+			}, nil, nil, nil, nil, nil, nil, nil,
 			func(opts *component.SystemOptions) *component.SystemOptions {
 				expectedOpts := defaultSystemOptions(opts)
 				expectedOpts.AppAffinity = testSystemAppAffinity()
@@ -533,7 +551,7 @@ func TestGetSystemOptionsProvider(t *testing.T) {
 				apimanager.Spec.System.SidekiqSpec.Tolerations = testSystemSidekiqTolerations()
 				apimanager.Spec.System.SphinxSpec.Tolerations = testSystemSphinxTolerations()
 				return apimanager
-			}, nil, nil, nil, nil, nil, nil,
+			}, nil, nil, nil, nil, nil, nil, nil,
 			func(opts *component.SystemOptions) *component.SystemOptions {
 				expectedOpts := defaultSystemOptions(opts)
 				expectedOpts.AppTolerations = testSystemAppTolerations()
@@ -551,7 +569,7 @@ func TestGetSystemOptionsProvider(t *testing.T) {
 				apimanager.Spec.System.SidekiqSpec.Resources = testSystemSidekiqCustomResourceRequirements()
 				apimanager.Spec.System.SphinxSpec.Resources = testSystemSphinxCustomResourceRequirements()
 				return apimanager
-			}, nil, nil, nil, nil, nil, nil,
+			}, nil, nil, nil, nil, nil, nil, nil,
 			func(opts *component.SystemOptions) *component.SystemOptions {
 				expectedOpts := defaultSystemOptions(opts)
 				expectedOpts.AppMasterContainerResourceRequirements = testSystemMasterContainerCustomResourceRequirements()
@@ -572,7 +590,7 @@ func TestGetSystemOptionsProvider(t *testing.T) {
 				apimanager.Spec.System.SidekiqSpec.Resources = testSystemSidekiqCustomResourceRequirements()
 				apimanager.Spec.System.SphinxSpec.Resources = testSystemSphinxCustomResourceRequirements()
 				return apimanager
-			}, nil, nil, nil, nil, nil, nil,
+			}, nil, nil, nil, nil, nil, nil, nil,
 			func(opts *component.SystemOptions) *component.SystemOptions {
 				expectedOpts := defaultSystemOptions(opts)
 				expectedOpts.AppMasterContainerResourceRequirements = testSystemMasterContainerCustomResourceRequirements()
@@ -583,6 +601,47 @@ func TestGetSystemOptionsProvider(t *testing.T) {
 				return expectedOpts
 			},
 		},
+		{"WithCustomMailFromAddress",
+			func() *appsv1alpha1.APIManager {
+				apimanager := basicApimanagerSpecTestSystemOptions()
+				return apimanager
+			}, nil, nil, nil, nil, nil, nil, getSMTPSecretWithCustomSMTPAddress("customaddress@customdomain.com"),
+			func(opts *component.SystemOptions) *component.SystemOptions {
+				expectedOpts := defaultSystemOptions(opts)
+				exampleFromAddress := "customaddress@customdomain.com"
+				expectedOpts.SmtpSecretOptions.FromAddress = &exampleFromAddress
+				return expectedOpts
+			},
+		},
+		{"WithExplicitelyEmptyMailFromAddress",
+			func() *appsv1alpha1.APIManager {
+				apimanager := basicApimanagerSpecTestSystemOptions()
+				return apimanager
+			}, nil, nil, nil, nil, nil, nil, getSMTPSecretWithCustomSMTPAddress(""),
+			func(opts *component.SystemOptions) *component.SystemOptions {
+				expectedOpts := defaultSystemOptions(opts)
+				emptyStr := ""
+				expectedOpts.SmtpSecretOptions.FromAddress = &emptyStr
+				return expectedOpts
+			},
+		},
+		{"WithNoEmptyMailFromAddress",
+			// When no SMTP From mail address is defined in the secret then
+			// the corresponding SystemOption option is set as the empty string.
+			// Then in the component if the option is empty or nil the corresponding
+			// system environment variable will not be set
+			func() *appsv1alpha1.APIManager {
+				apimanager := basicApimanagerSpecTestSystemOptions()
+
+				return apimanager
+			}, nil, nil, nil, nil, nil, nil, nil,
+			func(opts *component.SystemOptions) *component.SystemOptions {
+				expectedOpts := defaultSystemOptions(opts)
+				emptyStr := ""
+				expectedOpts.SmtpSecretOptions.FromAddress = &emptyStr
+				return expectedOpts
+			},
+		},
 	}
 
 	for _, tc := range cases {
@@ -606,6 +665,9 @@ func TestGetSystemOptionsProvider(t *testing.T) {
 			if tc.systemMasterApicastSecret != nil {
 				objs = append(objs, tc.systemMasterApicastSecret)
 			}
+			if tc.systemSMTPSecret != nil {
+				objs = append(objs, tc.systemSMTPSecret)
+			}
 			cl := fake.NewFakeClient(objs...)
 			optsProvider := NewSystemOptionsProvider(tc.apimanagerFactory(), namespace, cl)
 			opts, err := optsProvider.GetSystemOptions()
