@@ -4,14 +4,15 @@ import (
 	"context"
 	"fmt"
 	"reflect"
-	"strconv"
+
+	commonapps "github.com/3scale/3scale-operator/apis/apps"
+	appsv1alpha1 "github.com/3scale/3scale-operator/apis/apps/v1alpha1"
 
 	"github.com/3scale/3scale-operator/pkg/3scale/amp/component"
-	appsv1alpha1 "github.com/3scale/3scale-operator/pkg/apis/apps/v1alpha1"
+	"github.com/3scale/3scale-operator/pkg/common"
 	"github.com/3scale/3scale-operator/pkg/helper"
 	"github.com/3scale/3scale-operator/pkg/reconcilers"
 
-	monitoringv1 "github.com/coreos/prometheus-operator/pkg/apis/monitoring/v1"
 	"github.com/go-logr/logr"
 	"github.com/google/go-cmp/cmp"
 	appsv1 "github.com/openshift/api/apps/v1"
@@ -35,17 +36,43 @@ func NewUpgradeApiManager(b *reconcilers.BaseReconciler, apiManager *appsv1alpha
 }
 
 func (u *UpgradeApiManager) Upgrade() (reconcile.Result, error) {
-	res, err := u.upgradeImages()
+	res, err := u.deleteAPIManagerOwnerReferencesFromSecrets()
 	if err != nil {
-		return res, fmt.Errorf("Upgrading images: %w", err)
+		return res, fmt.Errorf("Deleting secrets APIManager owner references: %w", err)
 	}
 	if res.Requeue {
 		return res, nil
 	}
 
-	res, err = u.upgradeMonitoringSettings()
+	res, err = u.upgradeSystemAppUserSessionTTL()
 	if err != nil {
-		return res, fmt.Errorf("Upgrading monitoring settings: %w", err)
+		return res, fmt.Errorf("Upgrading system app user session ttl: %w", err)
+	}
+
+	res, err = u.upgradeBackendRouteEnv()
+	if err != nil {
+		return res, fmt.Errorf("Upgrading backend route env vars: %w", err)
+	}
+	if res.Requeue {
+		return res, nil
+	}
+
+	res, err = u.upgradeZyncPodTemplateAnnotations()
+	if err != nil {
+		return res, fmt.Errorf("Upgrading Zync DC PodTemplate: %w", err)
+	}
+	if res.Requeue {
+		return res, nil
+	}
+
+	res, err = u.upgradeBackendCronDeploymentConfigMemoryLimits()
+	if res.Requeue || err != nil {
+		return res, err
+	}
+
+	res, err = u.upgradeImages()
+	if err != nil {
+		return res, fmt.Errorf("Upgrading images: %w", err)
 	}
 	if res.Requeue {
 		return res, nil
@@ -132,7 +159,7 @@ func (u *UpgradeApiManager) upgradeDeploymentConfigs() (reconcile.Result, error)
 }
 
 func (u *UpgradeApiManager) upgradeAPIcastDeploymentConfigs() (reconcile.Result, error) {
-	apicast, err := Apicast(u.apiManager)
+	apicast, err := Apicast(u.apiManager, u.Client())
 	if err != nil {
 		return reconcile.Result{}, err
 	}
@@ -174,6 +201,63 @@ func (u *UpgradeApiManager) upgradeBackendDeploymentConfigs() (reconcile.Result,
 	return reconcile.Result{}, nil
 }
 
+func (u *UpgradeApiManager) backendCronResourcesOverriden() bool {
+	return u.apiManager.Spec.Backend != nil && u.apiManager.Spec.Backend.CronSpec != nil &&
+		u.apiManager.Spec.Backend.CronSpec.Resources != nil
+}
+
+func (u *UpgradeApiManager) upgradeBackendCronDeploymentConfigMemoryLimits() (reconcile.Result, error) {
+	backend, err := Backend(u.apiManager, u.Client())
+	if err != nil {
+		return reconcile.Result{}, err
+	}
+	desired := backend.CronDeploymentConfig()
+
+	// We only upgrade the memory limits when resource requirements flag is enabled
+	// and the user has not intentionally overriden them at backend-cron level
+	if (u.apiManager.Spec.ResourceRequirementsEnabled != nil && *u.apiManager.Spec.ResourceRequirementsEnabled) && !u.backendCronResourcesOverriden() {
+		existing := &appsv1.DeploymentConfig{}
+		err := u.Client().Get(context.TODO(), types.NamespacedName{Name: desired.Name, Namespace: u.apiManager.Namespace}, existing)
+		if err != nil {
+			return reconcile.Result{}, err
+		}
+
+		changed, err := u.ensureBackendCronMemoryLimit(desired, existing)
+		if err != nil {
+			return reconcile.Result{}, err
+		}
+		if changed {
+			u.Logger().Info(fmt.Sprintf("Upgrading '%s' DeploymentConfig Resource Memory Limits", desired.Name))
+			return reconcile.Result{Requeue: true}, u.UpdateResource(existing)
+		}
+
+	}
+	return reconcile.Result{}, nil
+}
+
+func (u *UpgradeApiManager) ensureBackendCronMemoryLimit(desired, existing *appsv1.DeploymentConfig) (bool, error) {
+	if len(existing.Spec.Template.Spec.Containers) != 1 {
+		return false, fmt.Errorf("Existing DeploymentConfig %s spec.template.spec.containers length is %d, should be 1",
+			existing.Name, len(existing.Spec.Template.Spec.Containers))
+	}
+	if len(desired.Spec.Template.Spec.Containers) != 1 {
+		return false, fmt.Errorf("Desired DeploymentConfig %s spec.template.spec.containers length is %d, should be 1",
+			existing.Name, len(desired.Spec.Template.Spec.Containers))
+	}
+
+	existingResources := &existing.Spec.Template.Spec.Containers[0].Resources
+	desiredResources := &desired.Spec.Template.Spec.Containers[0].Resources
+	existingLimits := existingResources.Limits
+	desiredLimits := desiredResources.Limits
+
+	changed := false
+	if existingLimits.Memory().Cmp(*desiredLimits.Memory()) != 0 {
+		existing.Spec.Template.Spec.Containers[0].Resources.Limits[v1.ResourceMemory] = desiredLimits.Memory().DeepCopy()
+		changed = true
+	}
+	return changed, nil
+}
+
 func (u *UpgradeApiManager) upgradeZyncDeploymentConfigs() (reconcile.Result, error) {
 	zync, err := Zync(u.apiManager, u.Client())
 	if err != nil {
@@ -190,9 +274,11 @@ func (u *UpgradeApiManager) upgradeZyncDeploymentConfigs() (reconcile.Result, er
 		return res, err
 	}
 
-	res, err = u.upgradeDeploymentConfigImageChangeTrigger(zync.DatabaseDeploymentConfig())
-	if res.Requeue || err != nil {
-		return res, err
+	if !u.apiManager.IsZyncExternalDatabaseEnabled() {
+		res, err = u.upgradeDeploymentConfigImageChangeTrigger(zync.DatabaseDeploymentConfig())
+		if res.Requeue || err != nil {
+			return res, err
+		}
 	}
 
 	return reconcile.Result{}, nil
@@ -216,24 +302,86 @@ func (u *UpgradeApiManager) upgradeBackendRedisDeploymentConfig() (reconcile.Res
 	if err != nil {
 		return reconcile.Result{}, err
 	}
+	desired := redis.BackendDeploymentConfig()
 
-	res, err := u.upgradeDeploymentConfigImageChangeTrigger(redis.BackendDeploymentConfig())
-	if res.Requeue || err != nil {
-		return res, err
+	existing := &appsv1.DeploymentConfig{}
+	err = u.Client().Get(context.TODO(), types.NamespacedName{Name: desired.Name, Namespace: u.apiManager.Namespace}, existing)
+	if err != nil {
+		return reconcile.Result{}, err
+	}
+
+	changed := false
+	tmpChanged, err := u.ensureDeploymentConfigImageChangeTrigger(desired, existing)
+	if err != nil {
+		return reconcile.Result{}, err
+	}
+	changed = changed || tmpChanged
+
+	tmpChanged, err = u.ensureRedisCommand(desired, existing)
+	if err != nil {
+		return reconcile.Result{}, err
+	}
+	changed = changed || tmpChanged
+
+	tmpChanged, err = u.ensureRedisPodTemplateLabels(desired, existing)
+	if err != nil {
+		return reconcile.Result{}, err
+	}
+	changed = changed || tmpChanged
+
+	if changed {
+		return reconcile.Result{Requeue: true}, u.UpdateResource(existing)
 	}
 
 	return reconcile.Result{}, nil
 }
 
+func (u *UpgradeApiManager) ensureRedisPodTemplateLabels(desired, existing *appsv1.DeploymentConfig) (bool, error) {
+	changed := false
+	existingPodTemplateLabels := &existing.Spec.Template.Labels
+	desiredPodTemplateLabels := desired.Spec.Template.Labels
+	helper.MergeMapStringString(&changed, existingPodTemplateLabels, desiredPodTemplateLabels)
+	if changed {
+		u.Logger().V(1).Info(fmt.Sprintf("%s DeploymentConfig's PodTemplate labels changed", desired.Name))
+	}
+
+	return changed, nil
+}
+
 func (u *UpgradeApiManager) upgradeSystemRedisDeploymentConfig() (reconcile.Result, error) {
 	redis, err := Redis(u.apiManager, u.Client())
 	if err != nil {
 		return reconcile.Result{}, err
 	}
+	desired := redis.SystemDeploymentConfig()
 
-	res, err := u.upgradeDeploymentConfigImageChangeTrigger(redis.SystemDeploymentConfig())
-	if res.Requeue || err != nil {
-		return res, err
+	existing := &appsv1.DeploymentConfig{}
+	err = u.Client().Get(context.TODO(), types.NamespacedName{Name: desired.Name, Namespace: u.apiManager.Namespace}, existing)
+	if err != nil {
+		return reconcile.Result{}, err
+	}
+
+	changed := false
+	tmpChanged, err := u.ensureDeploymentConfigImageChangeTrigger(desired, existing)
+	if err != nil {
+		return reconcile.Result{}, err
+	}
+	changed = changed || tmpChanged
+
+	tmpChanged, err = u.ensureRedisCommand(desired, existing)
+	if err != nil {
+		return reconcile.Result{}, err
+	}
+	changed = changed || tmpChanged
+
+	tmpChanged, err = u.ensureRedisPodTemplateLabels(desired, existing)
+	if err != nil {
+		return reconcile.Result{}, err
+	}
+	changed = changed || tmpChanged
+
+	if changed {
+		return reconcile.Result{Requeue: true}, u.UpdateResource(existing)
 	}
 
 	return reconcile.Result{}, nil
@@ -318,6 +466,25 @@ func (u *UpgradeApiManager) upgradeDeploymentConfigImageChangeTrigger(desired *a
 	return reconcile.Result{}, nil
 }
 
+func (u *UpgradeApiManager) ensureRedisCommand(desired, existing *appsv1.DeploymentConfig) (bool, error) {
+	if len(existing.Spec.Template.Spec.Containers) == 0 {
+		return false, fmt.Errorf("DeploymentConfig %s spec.template.spec.containers length is %d, should be 1", existing.Name, len(existing.Spec.Template.Spec.Containers))
+
+	}
+	if len(desired.Spec.Template.Spec.Containers) == 0 {
+		return false, fmt.Errorf("DeploymentConfig %s spec.template.spec.containers length is %d, should be 1", desired.Name, len(desired.Spec.Template.Spec.Containers))
+	}
+
+	changed := false
+	if !reflect.DeepEqual(existing.Spec.Template.Spec.Containers[0].Command, desired.Spec.Template.Spec.Containers[0].Command) {
+		existing.Spec.Template.Spec.Containers[0].Command = desired.Spec.Template.Spec.Containers[0].Command
+		u.Logger().V(1).Info(fmt.Sprintf("DeploymentConfig %s container command changed", desired.Name))
+		changed = true
+	}
+
+	return changed, nil
+}
+
 func (u *UpgradeApiManager) ensureDeploymentConfigImageChangeTrigger(desired, existing *appsv1.DeploymentConfig) (bool, error) {
 	desiredDeploymentTriggerImageChangePos, err := u.findDeploymentTriggerOnImageChange(desired.Spec.Triggers)
 	if err != nil {
@@ -407,172 +574,246 @@ func (u *UpgradeApiManager) findDeploymentTriggerOnImageChange(triggerPolicies [
 	return result, nil
 }
 
-func (u *UpgradeApiManager) upgradeMonitoringSettings() (reconcile.Result, error) {
-	updated := false
-
-	// Port and environment variable are exposed in DC for monitoring
-	updatedTmp, err := u.ensureSystemAppMonitoringSettings()
-	if err != nil {
-		return reconcile.Result{}, err
-	}
-	updated = updated || updatedTmp
-
-	// We explicitely set the metrics environment variable in system-sidekiq
-	// for clarity
-	updatedTmp, err = u.ensureSystemSidekiqMonitoringSettings()
-	if err != nil {
-		return reconcile.Result{}, err
-	}
-	updated = updated || updatedTmp
-
-	// Apicast rules
-	apicast, err := Apicast(u.apiManager)
-	if err != nil {
-		return reconcile.Result{}, err
-	}
-	updatedTmp, err = u.upgradePrometheusRules(apicast.ApicastPrometheusRules())
-	if err != nil {
-		return reconcile.Result{}, err
-	}
-	updated = updated || updatedTmp
-
-	// System Sidekiq rules
+func (u *UpgradeApiManager) upgradeSystemAppUserSessionTTL() (reconcile.Result, error) {
 	system, err := System(u.apiManager, u.Client())
 	if err != nil {
 		return reconcile.Result{}, err
 	}
-	updatedTmp, err = u.upgradePrometheusRules(system.SystemSidekiqPrometheusRules())
+	baseAPIManagerLogicReconciler := NewBaseAPIManagerLogicReconciler(u.BaseReconciler, u.apiManager)
+
+	// SystemApp Secret
+	err = baseAPIManagerLogicReconciler.ReconcileSecret(system.AppSecret(), reconcilers.DefaultsOnlySecretMutator)
 	if err != nil {
 		return reconcile.Result{}, err
 	}
-	updated = updated || updatedTmp
 
-	return reconcile.Result{Requeue: updated}, nil
+	res, err := u.upgradeSystemAppUserSessionTTLEnv(system.AppDeploymentConfig())
+	return res, err
 }
 
-func (u *UpgradeApiManager) ensureSystemSidekiqMonitoringSettings() (bool, error) {
-	existing := &appsv1.DeploymentConfig{}
-	err := u.Client().Get(context.TODO(), types.NamespacedName{Name: component.SystemSidekiqName, Namespace: u.apiManager.Namespace}, existing)
+func (u *UpgradeApiManager) upgradeBackendRouteEnv() (reconcile.Result, error) {
+	system, err := System(u.apiManager, u.Client())
 	if err != nil {
-		return false, err
+		return reconcile.Result{}, err
 	}
 
-	if len(existing.Spec.Template.Spec.Containers) != 1 {
-		return false, fmt.Errorf("DeploymentConfig %s spec.template.spec.containers length is %d, should be 1",
-			component.SystemSidekiqName, len(existing.Spec.Template.Spec.Containers))
+	res, err := u.upgradeSystemAppBackendRouteEnv(system.AppDeploymentConfig())
+	if res.Requeue || err != nil {
+		return res, err
 	}
 
-	update := false
-	existingEnvVarIdx := helper.FindEnvVar(existing.Spec.Template.Spec.Containers[0].Env, component.SystemSidekiqPrometheusExporterPortEnvVarName)
-	if existingEnvVarIdx < 0 {
-		existing.Spec.Template.Spec.Containers[0].Env = append(
-			existing.Spec.Template.Spec.Containers[0].Env,
-			helper.EnvVarFromValue(component.SystemSidekiqPrometheusExporterPortEnvVarName, strconv.Itoa(component.SystemSidekiqMetricsPort)),
-		)
-		update = true
+	res, err = u.upgradeSidekiqBackendRouteEnv(system.SidekiqDeploymentConfig())
+	if res.Requeue || err != nil {
+		return res, err
 	}
 
-	if update {
-		u.Logger().Info(fmt.Sprintf("Adding prometheus metrics environment variable to DC %s", component.SystemSidekiqName))
-		err = u.UpdateResource(existing)
-		if err != nil {
-			return false, err
-		}
-	}
-
-	return update, nil
+	return reconcile.Result{}, nil
 }
 
-func (u *UpgradeApiManager) ensureSystemAppMonitoringSettings() (bool, error) {
+func (u *UpgradeApiManager) upgradeSystemAppUserSessionTTLEnv(desired *appsv1.DeploymentConfig) (reconcile.Result, error) {
 	existing := &appsv1.DeploymentConfig{}
-	err := u.Client().Get(context.TODO(), types.NamespacedName{Name: component.SystemAppDeploymentName, Namespace: u.apiManager.Namespace}, existing)
+	err := u.Client().Get(context.TODO(), types.NamespacedName{Name: desired.Name, Namespace: u.apiManager.Namespace}, existing)
 	if err != nil {
-		return false, err
+		return reconcile.Result{}, err
 	}
 
 	if len(existing.Spec.Template.Spec.Containers) != 3 {
-		return false, fmt.Errorf("DeploymentConfig %s spec.template.spec.containers length is %d, should be 3",
+		return reconcile.Result{}, fmt.Errorf("DeploymentConfig %s spec.template.spec.containers length is %d, should be 3",
+			existing.Name, len(existing.Spec.Template.Spec.Containers))
+	}
+	desiredName := common.ObjectInfo(desired)
+
+	update := false
+	for idx := 0; idx < 3; idx++ {
+		existingContainer := &existing.Spec.Template.Spec.Containers[idx]
+		desiredContainer := &desired.Spec.Template.Spec.Containers[idx]
+		desiredUserSessionTTLEnvVarIdx := helper.FindEnvVar(desiredContainer.Env, component.SystemSecretSystemAppUserSessionTTLFieldName)
+		if desiredUserSessionTTLEnvVarIdx < 0 {
+			return reconcile.Result{}, fmt.Errorf("%s desired spec.template.spec.containers env var '%s' does not exist", desiredName, component.SystemSecretSystemAppUserSessionTTLFieldName)
+		}
+		tmpUpdate := helper.EnsureEnvVar(desiredContainer.Env[desiredUserSessionTTLEnvVarIdx], &existingContainer.Env)
+		update = update || tmpUpdate
+	}
+
+	if update {
+		u.Logger().Info(fmt.Sprintf("Upgrading USER_SESSION_TTL environment variable to DC %s", existing.Name))
+		err = u.UpdateResource(existing)
+		if err != nil {
+			return reconcile.Result{}, err
+		}
+	}
+
+	return reconcile.Result{Requeue: update}, nil
+}
+
+func (u *UpgradeApiManager) upgradeSystemAppBackendRouteEnv(desired *appsv1.DeploymentConfig) (reconcile.Result, error) {
+	existing := &appsv1.DeploymentConfig{}
+	err := u.Client().Get(context.TODO(), types.NamespacedName{Name: desired.Name, Namespace: u.apiManager.Namespace}, existing)
+	if err != nil {
+		return reconcile.Result{}, err
+	}
+
+	if len(existing.Spec.Template.Spec.Containers) != 3 {
+		return reconcile.Result{}, fmt.Errorf("DeploymentConfig %s spec.template.spec.containers length is %d, should be 3",
 			existing.Name, len(existing.Spec.Template.Spec.Containers))
 	}
 
+	desiredName := common.ObjectInfo(desired)
+
 	update := false
-	for idx := range existing.Spec.Template.Spec.Containers {
-		container := &existing.Spec.Template.Spec.Containers[idx]
-		var containerPrometheusMetricsPort int
-		var containerPrometheusMetricsPortName string
-		switch containerName := container.Name; {
-		case containerName == component.SystemAppMasterContainerName:
-			containerPrometheusMetricsPort = component.SystemAppMasterContainerPrometheusPort
-			containerPrometheusMetricsPortName = component.SystemAppMasterContainerMetricsPortName
-		case containerName == component.SystemAppProviderContainerName:
-			containerPrometheusMetricsPort = component.SystemAppProviderContainerPrometheusPort
-			containerPrometheusMetricsPortName = component.SystemAppProviderContainerMetricsPortName
-		case containerName == component.SystemAppDeveloperContainerName:
-			containerPrometheusMetricsPort = component.SystemAppDeveloperContainerPrometheusPort
-			containerPrometheusMetricsPortName = component.SystemAppDeveloperContainerMetricsPortName
-		default:
-			return false, fmt.Errorf("DeploymentConfig '%s' has unrecognized container name '%s'", existing.Name, containerName)
+	for idx := 0; idx < 3; idx++ {
+		existingContainer := &existing.Spec.Template.Spec.Containers[idx]
+		desiredContainer := &desired.Spec.Template.Spec.Containers[idx]
+		desiredBackendRouteEnvVarIdx := helper.FindEnvVar(desiredContainer.Env, "BACKEND_ROUTE")
+		if desiredBackendRouteEnvVarIdx < 0 {
+			return reconcile.Result{}, fmt.Errorf("%s desired spec.template.spec.containers env var '%s' does not exist", desiredName, "BACKEND_ROUTE")
+		}
+		tmpUpdate := helper.EnsureEnvVar(desiredContainer.Env[desiredBackendRouteEnvVarIdx], &existingContainer.Env)
+		update = update || tmpUpdate
+	}
+
+	// Pre hook pod env vars
+	desiredBackendRouteEnvVarIdx := helper.FindEnvVar(desired.Spec.Strategy.RollingParams.Pre.ExecNewPod.Env, "BACKEND_ROUTE")
+	if desiredBackendRouteEnvVarIdx < 0 {
+		return reconcile.Result{}, fmt.Errorf("%s desired spec.strategy.rollingparams.pre.execnewpod env var '%s' does not exist", desiredName, "BACKEND_ROUTE")
+	}
+	tmpUpdate := helper.EnsureEnvVar(desired.Spec.Strategy.RollingParams.Pre.ExecNewPod.Env[desiredBackendRouteEnvVarIdx], &existing.Spec.Strategy.RollingParams.Pre.ExecNewPod.Env)
+	update = update || tmpUpdate
+
+	if update {
+		u.Logger().Info(fmt.Sprintf("Upgrading BACKEND_ROUTE environment variable to DC %s", existing.Name))
+		err = u.UpdateResource(existing)
+		if err != nil {
+			return reconcile.Result{}, err
+		}
+	}
+
+	return reconcile.Result{Requeue: update}, nil
+}
+
+func (u *UpgradeApiManager) upgradeSidekiqBackendRouteEnv(desired *appsv1.DeploymentConfig) (reconcile.Result, error) {
+	existing := &appsv1.DeploymentConfig{}
+	err := u.Client().Get(context.TODO(), types.NamespacedName{Name: desired.Name, Namespace: u.apiManager.Namespace}, existing)
+	if err != nil {
+		return reconcile.Result{}, err
+	}
+
+	if len(existing.Spec.Template.Spec.Containers) != 1 {
+		return reconcile.Result{}, fmt.Errorf("DeploymentConfig %s spec.template.spec.containers length is %d, should be 1",
+			existing.Name, len(existing.Spec.Template.Spec.Containers))
+	}
+
+	desiredName := common.ObjectInfo(desired)
+
+	existingContainer := &existing.Spec.Template.Spec.Containers[0]
+	desiredContainer := &desired.Spec.Template.Spec.Containers[0]
+	desiredBackendRouteEnvVarIdx := helper.FindEnvVar(desiredContainer.Env, "BACKEND_ROUTE")
+	if desiredBackendRouteEnvVarIdx < 0 {
+		return reconcile.Result{}, fmt.Errorf("%s desired spec.template.spec.containers env var '%s' does not exist", desiredName, "BACKEND_ROUTE")
+	}
+	update := helper.EnsureEnvVar(desiredContainer.Env[desiredBackendRouteEnvVarIdx], &existingContainer.Env)
+
+	if update {
+		u.Logger().Info(fmt.Sprintf("Upgrading BACKEND_ROUTE environment variable to DC %s", existing.Name))
+		err = u.UpdateResource(existing)
+		if err != nil {
+			return reconcile.Result{}, err
+		}
+	}
+	return reconcile.Result{Requeue: update}, nil
+}
+
+func (u *UpgradeApiManager) upgradeZyncPodTemplateAnnotations() (reconcile.Result, error) {
+	zync, err := Zync(u.apiManager, u.Client())
+	if err != nil {
+		return reconcile.Result{}, err
+	}
+
+	desired := zync.DeploymentConfig()
+	existing := &appsv1.DeploymentConfig{}
+	err = u.Client().Get(context.TODO(), types.NamespacedName{Name: desired.Name, Namespace: u.apiManager.Namespace}, existing)
+	if err != nil {
+		return reconcile.Result{}, err
+	}
+
+	if existing.Spec.Template.Annotations == nil {
+		existing.Spec.Template.Annotations = map[string]string{}
+	}
+	update := false
+
+	for desiredAnnotationKey, desiredAnnotationVal := range desired.Spec.Template.Annotations {
+		existingAnnotationVal, ok := existing.Spec.Template.Annotations[desiredAnnotationKey]
+		if !ok || existingAnnotationVal != desiredAnnotationVal {
+			existing.Spec.Template.Annotations[desiredAnnotationKey] = desiredAnnotationVal
+			update = true
 		}
 
-		existingEnvVarIdx := helper.FindEnvVar(container.Env, component.SystemAppPrometheusExporterPortEnvVarName)
-		if existingEnvVarIdx < 0 {
-			container.Env = append(
-				container.Env,
-				helper.EnvVarFromValue(component.SystemAppPrometheusExporterPortEnvVarName, strconv.Itoa(containerPrometheusMetricsPort)),
-			)
-			update = true
-		}
-		if _, ok := helper.FindContainerPortByName(container.Ports, containerPrometheusMetricsPortName); !ok {
-			container.Ports = append(container.Ports, v1.ContainerPort{Name: containerPrometheusMetricsPortName, ContainerPort: int32(containerPrometheusMetricsPort), Protocol: v1.ProtocolTCP})
-			update = true
+		if existing.Annotations != nil {
+			if _, ok := existing.Annotations[desiredAnnotationKey]; ok {
+				delete(existing.Annotations, desiredAnnotationKey)
+				update = true
+			}
 		}
 	}
 
 	if update {
-		u.Logger().Info(fmt.Sprintf("Adding prometheus metrics environment variable and ports to DC %s", existing.Name))
+		u.Logger().Info(fmt.Sprintf("Upgrading zync DC %s PodTemplate annotations", existing.Name))
 		err = u.UpdateResource(existing)
 		if err != nil {
-			return false, err
+			return reconcile.Result{}, err
 		}
 	}
 
-	return update, nil
-}
-
-func (u *UpgradeApiManager) upgradePrometheusRules(desired *monitoringv1.PrometheusRule) (bool, error) {
-	if !u.apiManager.IsMonitoringEnabled() {
-		return false, nil
-	}
-
-	existing := &monitoringv1.PrometheusRule{}
-	err := u.Client().Get(context.TODO(), types.NamespacedName{Name: desired.Name, Namespace: u.apiManager.Namespace}, existing)
-	if err != nil {
-		return false, err
-	}
-
-	changed := u.ensurePrometheusRulesEqual(desired, existing)
-	if changed {
-		u.Logger().Info(fmt.Sprintf("Upgrading prometheus rules: %s", existing.Name))
-		err = u.UpdateResource(existing)
-		if err != nil {
-			return true, err
-		}
-	}
-
-	return changed, nil
-}
-
-func (u *UpgradeApiManager) ensurePrometheusRulesEqual(desired, existing *monitoringv1.PrometheusRule) bool {
-	if !reflect.DeepEqual(existing.Spec, desired.Spec) {
-		diff := cmp.Diff(existing.Spec, desired.Spec)
-		u.Logger().V(1).Info(fmt.Sprintf("%s prometheus rules changed: %s", desired.Name, diff))
-		existing.Spec = desired.Spec
-		return true
-	}
-
-	return false
+	return reconcile.Result{Requeue: update}, nil
 }
 
 func (u *UpgradeApiManager) Logger() logr.Logger {
 	return u.logger
 }
+
+func (u *UpgradeApiManager) deleteAPIManagerOwnerReferencesFromSecrets() (reconcile.Result, error) {
+	secretsToUpdate := []string{
+		component.BackendSecretInternalApiSecretName,
+		component.BackendSecretBackendListenerSecretName,
+		component.BackendSecretBackendRedisSecretName,
+		component.SystemSecretSystemAppSecretName,
+		component.SystemSecretSystemDatabaseSecretName,
+		component.SystemSecretSystemEventsHookSecretName,
+		component.SystemSecretSystemMasterApicastSecretName,
+		component.SystemSecretSystemMemcachedSecretName,
+		component.SystemSecretSystemRecaptchaSecretName,
+		component.SystemSecretSystemRedisSecretName,
+		component.SystemSecretSystemSeedSecretName,
+		component.SystemSecretSystemSMTPSecretName,
+		component.ZyncSecretName,
+	}
+
+	someChanged := false
+	for _, secretName := range secretsToUpdate {
+		existing := &v1.Secret{}
+		err := u.Client().Get(context.TODO(), types.NamespacedName{Name: secretName, Namespace: u.apiManager.Namespace}, existing)
+		if err != nil {
+			return reconcile.Result{}, err
+		}
+
+		apimanagerOwnerReferenceIdx := -1
+		for currIdx, secretOwnerReference := range existing.OwnerReferences {
+			if secretOwnerReference.Kind == commonapps.APIManagerKind &&
+				secretOwnerReference.Controller != nil && *secretOwnerReference.Controller {
+				apimanagerOwnerReferenceIdx = currIdx
+			}
+		}
+
+		if apimanagerOwnerReferenceIdx != -1 {
+			u.Logger().Info(fmt.Sprintf("Removing APIManager OwnerReference from Secret %s", existing.Name))
+			existing.OwnerReferences = append(existing.OwnerReferences[:apimanagerOwnerReferenceIdx], existing.OwnerReferences[apimanagerOwnerReferenceIdx+1:]...)
+			err = u.UpdateResource(existing)
+			if err != nil {
+				return reconcile.Result{}, err
+			}
+			someChanged = true
+		}
+	}
+
+	return reconcile.Result{Requeue: someChanged}, nil
+}
